import { query } from "./lib/clickhouse.js";
import { publicProcedure, RpcType, router } from "./trpc.js";
import {
  Object as ObjectT,
  String as StringT,
  Number as NumberT,
} from "@sinclair/typebox";

export const getChartData = publicProcedure
  .input(
    RpcType(
      ObjectT({
        underlying: StringT({ maxLength: 5 }),
        daysToFrontExpiration: NumberT(),
        daysBetweenFrontAndBackExpiration: NumberT(),
        lookbackPeriodStart: StringT({
          pattern: "[0-9]{4}-[0-9]{2}-[0-9]{2}",
        }),
        lookbackPeriodEnd: StringT({ pattern: "[0-9]{4}-[0-9]{2}-[0-9]{2}" }),
      })
    )
  )
  .query(async (opts) => {
    const {
      underlying,
      daysToFrontExpiration,
      daysBetweenFrontAndBackExpiration,
      lookbackPeriodStart,
      lookbackPeriodEnd,
    } = opts.input;
    return await query<[number, number, number]>(
      `
     SELECT
			 moniness as x,
			 FLOOR(price, 1) as y,
			 sum(number_of_quotes) as n
		 FROM calendar_stats
		 WHERE dte = ${daysToFrontExpiration}
		 AND moniness >= -0.05
		 AND moniness <= 0.05
		 AND span = ${daysBetweenFrontAndBackExpiration}
		 AND date >= '${lookbackPeriodStart}'
		 AND date <= '${lookbackPeriodEnd}'
		 GROUP BY x, y
		 ORDER BY x ASC, y ASC
	 `,
      "JSONEachRow"
    );
  });

export default router({
  getChartData,
});
