import { query } from "./lib/clickhouse.js";
import { publicProcedure, RpcType, router } from "./trpc.js";
import {
  Object as ObjectT,
  String as StringT,
  Number as NumberT,
} from "@sinclair/typebox";

/** Returns prices for all matching calendars (i.e. those with similar
 * characteristics to those given) */
export const getChartData = publicProcedure
  .input(
    RpcType(
      ObjectT({
        underlying: StringT({ maxLength: 5 }),
        daysToFrontExpiration: NumberT(),
        daysBetweenFrontAndBackExpiration: NumberT(),
        strikePercentageFromUnderlyingPriceRangeMin: NumberT(),
        strikePercentageFromUnderlyingPriceRangeMax: NumberT(),
        lookbackPeriodStart: StringT(),
        lookbackPeriodEnd: StringT(),
      })
    )
  )
  .query(async (opts) => {
    const {
      underlying,
      daysToFrontExpiration,
      daysBetweenFrontAndBackExpiration,
      strikePercentageFromUnderlyingPriceRangeMin,
      strikePercentageFromUnderlyingPriceRangeMax,
      lookbackPeriodStart,
      lookbackPeriodEnd,
    } = opts.input;
    return await query<[number, number]>(
      `
     SELECT
			 toUnixTimestamp(date) as x,
			 price as y
		 FROM calendar_stats
		 WHERE dte = ${daysToFrontExpiration}
		 AND moniness >= ${strikePercentageFromUnderlyingPriceRangeMin}
		 AND moniness <= ${strikePercentageFromUnderlyingPriceRangeMax}
		 AND span = ${daysBetweenFrontAndBackExpiration}
		 AND date >= '${lookbackPeriodStart}'
		 AND date <= '${lookbackPeriodEnd}'
	 `,
      "JSONEachRow"
    );
  });

export default router({
  getChartData,
});
